#! /usr/bin/env python3

from typing import Generator
import tree_sitter_c_sharp as tscsharp
from tree_sitter import Language, Parser, Tree, Node
import sys


CS_LANGUAGE = Language(tscsharp.language())
parser = Parser(CS_LANGUAGE)

code_in = open('/home/folkert/Projects/DotXT/State8086.cs', 'rb').read()
tree = parser.parse(code_in)

output = 'State8086.py'
fh = open(output, 'w')

def emit_raw(what):
    global fh
    fh.write(f'{what}\n')

def emit_empty():
    emit_raw('')

def emit(indentation, what):
    emit_raw(f'{indentation * " "}{what}')

def emit_comment(indentation, what):
    emit_raw(f'{indentation * " "}# {what}')

def dump(obj):
    for attr in dir(obj):
        print(f'obj.{attr} = {getattr(obj, attr)}', file=sys.stderr)

def find_child_by_type(obj, search_type):
    for child in obj.children:
        if child.type == search_type:
            return child
    return None

def emit_class_header(indentation, name):
    emit(indentation, f'class {name}:')

def emit_method_header(indentation, name):
    emit(indentation, f'def {name}:')

def emit_class__init(indentation, init_stuff):
    emit_method_header(indentation, '__init__(self)')
    for var in init_stuff:
        emit(indentation + 1, var)

def emit_decorator(indentation, name):
    emit(indentation, f'@{name}')

def emit_property(indentation, obj, name, class_):
    parent = 'self' if class_ != None else ''
    parentd = 'self.' if class_ != None else ''

    want_get = False
    want_set = False
    for i in obj.children:
        for j in i.children:
            if j.type == 'accessor_declaration':
                type_ = j.text.decode()
                if type_ == 'get;':
                    want_get = True
                if type_ == 'set;':
                    want_set = True

    if class_ != None:
        class_.append(f'{parentd}variable_{name} = None')
    else:
        emit(indentation + 1, f'variable_{name} = None')
    emit_empty()

    if want_get:
        emit_decorator(indentation + 1, 'property')
        emit_method_header(indentation + 1, name + f'({parent})')
        emit(indentation + 2, f'return {parentd}variable_{name}')
        emit_empty()

    if want_set:
        emit_decorator(indentation + 1, f'{name}.setter')
        if class_ != None:
            emit_method_header(indentation + 1, name + '(self, v)')
        else:
            emit_method_header(indentation + 1, name + '(v)')
        emit(indentation + 2, f'{parentd}variable_{name} = v')
        emit_empty()

def process_class(indentation, name, obj):
    emit_class_header(indentation, name)

    init_stuff = []

    for child in obj.children:
        if child.type == '{' or child.type == '}':
            continue

        if child.type == 'property_declaration':
            emit_property(indentation, child, find_child_by_type(child, 'identifier').text.decode(), init_stuff)
        #else:
        #    print(child.type, find_child_by_type(child, 'identifier').text.decode())

    if len(init_stuff) > 0:
        emit_class__init(indentation + 1, init_stuff)

def process_compilation_unit(indentation, children):
    print('process_compilation_unit', file=sys.stderr)

    for child in children:
        if child.type == 'using_directive':
            emit_comment(indentation, child.text.decode())
        elif child.type == 'class_declaration':
            class_name = find_child_by_type(child, 'identifier').text.decode()
            process_class(indentation, class_name, find_child_by_type(child, 'declaration_list'))
            #emit_comment(child.text.decode())
            #dump(child)
        #else:
        #    print(child.type, file=sys.stderr)

cursor = tree.walk()
indentation = 0
while True:
    print(cursor.node.type)
    if cursor.node.type == 'compilation_unit':
        process_compilation_unit(indentation, cursor.node.children)
    #else:
    #    print(cursor.node.type, dir(cursor.node))

    if cursor.goto_next_sibling() == False:
        break
